/**
 * Title:	Traces
 * Copyright:	Copyright (c) Jan-Pascal van Best
 * Email:	    janpascal@vanbest.org
 * @author	    Jan-Pascal van Best
 **************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package nl.tudelft.tbm.matrix;


import java.util.*;
import java.io.*;
import java.lang.*;
import java.text.*;

public class GaussJordan {
    SimpleMatrix A;
    double[][] a;
	int m;
	int n;

    int[] pivots;
    int rank = -1;

	public GaussJordan(Matrix A) {
        this.A = new SimpleMatrix(A);
        this.m = this.A.m;
        this.n = this.A.n;
        this.a = this.A.a;
        gaussJordan();
	}

    void gaussJordan() {
        pivots = new int[m];

        // First sweep the matrix
        System.out.print("Sweeping, i=");
        int pivot = 0;
        int i;
        for(i=0; i<m; i++){
            System.out.println(" "+i);
            // swap rows until we have a pivot
            while (pivot<n && a[i][pivot] == (double) 0) {
                for(int k=i; k<m; k++) {
                    if (a[k][pivot] != (double) 0) {
                        swapRow(i,k);
                        break;
                    }
                }
                if(a[i][pivot] == (double) 0) {
                    pivot++;
                }
            }
            System.out.println("pivot: (p="+pivot+")");
            pivots[i] = pivot;

            if(pivot==n) {
                // finished
                break;
            }

            // get leading one
            scaleRow(i, 1.0/a[i][pivot]);

            //get zeros above leading one
            for(int k=i-1; k>=0; k--)
                addRow(k,i,-1.0*a[k][pivot]);

            //get zeros below leading one
            for(int k=i+1; k<m; k++)
                addRow(k,i,-1.0*a[k][pivot]);

        } // end for loop

        rank = i;

        // Fill in rest of pivots
        for (int k=i; k<m; k++) pivots[k] = n;

        // Clean up "-0"s
        for (i=0; i<m; i++) {
            for (int j=0; j<n; j++) {
                if (a[i][j] == (double) -0.0) a[i][j] = (double) 0;
            }
        }
                              
        System.out.println("Rank: "+rank);
    }

}

