/**
 * Title:	Traces
 * Copyright:	Copyright (c) Jan-Pascal van Best
 * Email:	    janpascal@vanbest.org
 * @author	    Jan-Pascal van Best
 **************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package nl.tudelft.tbm.matrix;


import java.util.*;
import java.io.*;
import java.lang.*;

public abstract class SparseMatrix implements Matrix {
	int m;
	int n;

	int[][] nonzero;
    int[] rowLength;
	double[][] a;

	public SparseMatrix(int m, int n) {
		this.m = m;
		this.n = n;
		nonzero = new int[m][n];
		rowLength = new int[m];
		a = new double[m][n];
	}

	public SparseMatrix(double[][] A) {
		n = a.length;
		if (n==0) {
			m = 0;
		} else {
			m = a[0].length;
		}

		nonzero = new int[m][n];
		rowLength = new int[m];
		a = new double[m][n];

		for(int i=0; i<m; i++) {
			int k=0;
			for(int j=0; i<n; j++) {
				a[i][j] = A[i][j];
				if(a[i][j] != 0.) {
					nonzero[i][k] = j;
					k++;
				}
			}
			rowLength[i] = k;
		}
	}

    public int getRows() { return m; }
    public int getColumns() { return n; }

	public double get(int i, int j) {
		return a[i][j];
	}

	public void set(int i, int j, double value) {
		if (a[i][j] == 0.) {
			int k=0;
			while(nonzero[i][k]<k) k++;
        }

		a[i][j] = value;
    }

	public Matrix transpose() {
		return this;
	}

	// matrix algebra
	public Matrix times(Matrix other) {
        return this;
    }
	public Matrix times(double other) {
        return this;
    }
	public Matrix plus(Matrix other) {
        return this;
    }

	// elementary operations
	public void swapRow(int r1, int r2) {}
	public void scaleRow(int row, double factor) {}
	// r1 := r1 + factor * r2
	public void addRow(int r1, int r2, double factor) {}

	//
    public int rank() { return 0; }
}

