/**
 * Title:	Traces
 * Description: An analysis tool for traceroutes
 * Copyright:	Copyright (c) Jan-Pascal van Best
 * Email:	    janb@tbm.tudelft.nl
 * @author	    Jan-Pascal van Best
 **************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package nl.tudelft.tbm.matrix;


import java.io.*;
import java.lang.*;
import java.lang.reflect.*;
import java.net.*;
import java.util.*;
import java.util.logging.*;
import java.text.*;

public class Test {
    private static Logger logger = Logger.getLogger("nl.tudelft.tbm.matrix");

        static double[][] a = {
                {1,0,0,1,1,1,0,0},
                {1,0,0,1,1,0,1,0},
                {1,0,0,1,1,0,0,1},
                {0,1,0,1,1,1,0,0},
                {0,1,0,1,1,0,1,0},
                {0,1,0,1,1,0,0,1},
                {0,0,1,1,1,1,0,0},
                {0,0,1,1,1,0,1,0},
                {0,0,1,1,1,0,0,1},
                {1,1,0,1,1,1,1,0},
                {1,1,0,0,0,0,0,0}
            };
        static double[][] b = {
                {40},
                {45},
                {50},
                {40},
                {45},
                {50},
                {40},
                {45},
                {50},
                {55},
                {20}
            };
        static double[][] a2 = {
                {1,0,0,1,0,0},
                {1,0,0,0,1,0},
                {1,0,0,0,0,1},
                {0,1,0,1,0,0},
                {0,1,0,0,1,0},
                {0,1,0,0,0,1},
                {0,0,1,1,0,0},
                {0,0,1,0,1,0},
                {0,0,1,0,0,1},
            };
        static double[][] a3 = {
                {1,0,0,0,0,0},
                {0,1,0,0,0,0},
                {0,0,0,0,1,0},
                {0,0,0,0,0,1},
                {1,0,1,1,1,0},
                {0,1,1,1,1,0},
                {1,0,1,1,0,1},
                {0,1,1,1,0,1},
            };
        static double[][] b3 = {
                {10.001},
                {15.002},
                {30.001},
                {35.002},
                {85.001},
                {90.002},
                {90.001},
                {95.002},
            };
        static double[][] B3 = {
                {1,0,0,0,0,0},
                {0,1,0,0,0,0},
                {0,0,0,0,1,0},
                {0,0,0,0,0,1},
                {0,0,1,1,0,0},
                {0,0,1,-1,0,0}
        };
        static double[][] a4 = {
                {0,1,-3},
                {2,3,-1},
                {4,5,-2}
            };
        static double[][] t = {
                {1,0,0,0,0,0},
                {0,1,0,0,0,0},
                {0,0,1,1,0,0},
                {0,0,0,0,1,0},
                {0,0,0,0,0,1},
            };
        static double[][] ab = {
                {1,0,0,0,0,0},
                {0,1,0,0,0,0},
                {0,0,0,0,1,0},
                {0,0,0,0,0,1},
                {1,0,1,1,1,0},
                {0,1,1,1,1,0},
                {1,0,1,1,0,1},
                {0,1,1,1,0,1},
            };
        static double[] x0 = {-1,-1,-1,1,0,0,0,0};
        static double[] x1 = {-1,-1,-1,0,1,0,0,0};
        static double[] x2 = {-1,-1,-1,0,0,1,1,1};
        static double[] x3 = {1,0,0,1,1,1,0,0};
        static double[] x4 = {0,1,0,1,1,0,0,1};

    static void test1() {

/*
        SimpleMatrix B = new SimpleMatrix(6,12);
        for(int i=0; i<6; i++) {
            for(int j=0; j<6; j++) {
                B.set(i,j,B3[j][i]);
            }
            B.set(i,i+6, 1);
        }
        System.out.println("Original matrix");
        B.print(6,2);

        B.gaussJordan();
        System.out.println("Reduced matrix");
        B.print(6,2);
*/


        SimpleMatrix A = new SimpleMatrix(a);
        SimpleMatrix B = new SimpleMatrix(b);
        VectorSpace S = new VectorSpace(A);
        logger.info("Basis for row space: ");
        logger.info(S.toString(6,2));
        logger.info("rank:"+S.getDimension());
//        Matrix N = S.getNullBasis();
//        logger.info("null space:");
//        logger.info(N.toString(6,2));
//        if (!testNullSpace(A, N)) {
//            logger.severe("ERROR !!!! Null space incorrect!!! BUG BUG BUG");
//        }

//        SimpleMatrix B = new SimpleMatrix(B3);
//        SimpleMatrix Bt = (SimpleMatrix) B.transpose();
//        SimpleMatrix Cinv = S.makeNiceBasisIncludeNull(A);
        SimpleMatrix Cinv = S.makeNiceBasis(A);
        logger.info("Nice basis, not including null space: ");
        logger.info(Cinv.toString(6,2));
/*
        SimpleMatrix C = (SimpleMatrix) niceBasis.inverse();
        System.out.println("C");
        C.print(6,2);

        System.out.println("C^-1");
        SimpleMatrix Cinv = (SimpleMatrix) C.inverse();
        Cinv.print(6,2);
*/
/*
        SimpleMatrix AC = (SimpleMatrix) A.times(C);
        System.out.println("A.C: ");
        AC.print(6,2);
*/
        SimpleMatrix ACinv = (SimpleMatrix) A.times(Cinv);
        logger.info("A.Cinv: ");
        logger.info(ACinv.toString(6,2));

/*
        SimpleMatrix CinvAC = (SimpleMatrix) Cinv.times(AC);
        System.out.println("C-1.A.C: ");
        CinvAC.print(6,2);

        SimpleMatrix CACinv = (SimpleMatrix) C.times(A).times(Cinv);
        System.out.println("C.A.Cinv: ");
        CACinv.print(6,2);
*/

        SimpleMatrix Cx = (SimpleMatrix) NNLS.NNLS(ACinv, B);
//        System.out.println("C.x:");
//        Cx.print(6,2);

//        Jama.QRDecomposition qr = new Jama.QRDecomposition(simpleToJama(ACinv));
//        SimpleMatrix Cx = jamaToSimple(qr.solve(simpleToJama(B)));
        logger.info("C.x:");
        logger.info(Cx.toString(6,2));

        SimpleMatrix x = (SimpleMatrix) Cinv.times(Cx);
        logger.info("x:");
        logger.info(x.toString(6,2));

        Matrix Ax = A.times(x);
        logger.info("A.x:");
        logger.info(Ax.toString(6,2));







//        SimpleMatrix ACinv = (SimpleMatrix) CinvAC.inverse();
//        System.out.println("(Cinv.A.C)^-1: ");
//        CACinv.print(6,2);

//        testMatrix(new SimpleMatrix(a3), new SimpleMatrix(b3));

//        Log.redirectOut();
//		Log.setColumns(150);
/*
        SimpleMatrix A = new SimpleMatrix(a);
        A.print(5,1);
        VectorSpace S = new VectorSpace(A);
        S.A.print(5,1);

        System.out.println(S.contains(new SimpleMatrix(x0)));
        System.out.println(S.contains(new SimpleMatrix(x1)));
        System.out.println(S.contains(new SimpleMatrix(x2)));
        System.out.println(S.contains(new SimpleMatrix(x3)));
        System.out.println(S.contains(new SimpleMatrix(x4)));
*/
//        A.gaussJordan();
//        A.print(3,0);
/*
        Matrix B = new Matrix(b);
        Matrix T = new Matrix(t);

*/

/*
        System.out.println("T: ");
        T.print(2,0);

        Matrix TT = T.transpose();
        System.out.println("T': ");
        TT.print(2,0);

        Matrix ATT = A.times(TT);
        System.out.println("A.T': ");
        ATT.print(2,0);
        System.out.println("rank(A.T'): "+ATT.rank());
*/
    }

    public static boolean testNullSpace(Matrix A, Matrix N) {
        boolean okay = true;
        Matrix x = new SimpleMatrix(N.getColumns(), 1);
        for(int i=0; i<N.getRows(); i++) {
            for(int j=0; j<N.getColumns(); j++) {
                x.set(j,0, N.get(i,j));
            }
            Matrix result = A.times(x);
//            System.out.println("Result of "+i+"th null space basis vector:");
//            result.print(6,2);
            for(int j=0; j<N.getColumns(); j++) {
                if(result.get(j,0)!=(double)0 && result.get(j,0)!=(double)-0) okay=false;
            }
        }
        return okay;
    }

    static void test2() throws Exception {
        double[] row = {0,1,0,0,1,0,1,0,0,1,0,0};

        for(int i=1; i<=row.length; i++) {
            logger.info("Listing vectors with "+i+" ones");
            double[] v = new double[row.length];
            int place = -1;
            boolean ready = false;
            for (int j=0; j<i; j++) {
                place = VectorSpace.findNextOne(row,place+1);
                if (place>=v.length) {
                    ready = true;
                    break;
                }
                v[place] = (double) 1;
            }
            if(ready) break;
            do {
                StringBuffer s = new StringBuffer(v.length*4);
                for(int j=0; j<v.length; j++) {
                    s.append(" "+v[j]);
                }
                logger.info("Vector: "+s);
            } while(VectorSpace.nextBasisTryFromRow(row, v));
        }
    }

    static SimpleMatrix DGELSX(SimpleMatrix A, SimpleMatrix B) {
        int m = A.getRows();
        int n = A.getColumns();
        SimpleMatrix x = new SimpleMatrix(m,1);
        for(int j=0; j<m; j++) {
            x.a[j][0] = B.a[j][0];
        }
        int[] JPVT = new int[n];
        org.netlib.util.intW RANK = new org.netlib.util.intW(0);
        double[] work = new double[(Math.max( Math.min(m,n)+3*n, 2*Math.min(m,n)+1 ))];
        org.netlib.util.intW INFO = new org.netlib.util.intW(0);
        org.netlib.lapack.DGELSX.DGELSX(m, n, 1, A.a, x.a, JPVT, 1E-4, RANK, work, INFO);

        System.out.println("RANK: "+RANK.val);
        System.out.println("INFO: "+INFO.val);

        return x;
    }

    static public void testMatrix(SimpleMatrix A, SimpleMatrix B) {
        logger.info("A: ");
        logger.info(A.toString(8,3));

        logger.info("B: ");
        logger.info(B.toString(8,3));

        VectorSpace S = new VectorSpace(A);
        logger.info("S: ");
        logger.info(S.A.toString(8,3));

		logger.info("Calculating nice basis...");
		SimpleMatrix Cinv = S.makeNiceBasis(A);

        logger.info("Nice basis: ");
		logger.info(Cinv.toString(8,3));

        SimpleMatrix ACinv = (SimpleMatrix) A.times(Cinv);
        logger.info("A.Cinv: ");
        logger.info(ACinv.toString(8,3));

        SimpleMatrix Cx = (SimpleMatrix) NNLS.NNLS(ACinv, B);
        logger.info("C.x:");
        logger.info(Cx.toString(8,3));

        SimpleMatrix x = (SimpleMatrix) Cinv.times(Cx);
        logger.info("x:");
        logger.info(x.toString(8,3));

        Matrix Ax = A.times(x);
        logger.info("A.x:");
        logger.info(Ax.toString(8,3));
    }
/*
    static Jama.Matrix simpleToJama(SimpleMatrix A) {
        return Jama.Matrix.constructWithCopy(A.a);
    }

    static SimpleMatrix jamaToSimple(Jama.Matrix A) {
        return new SimpleMatrix(A.getArray());
    }
*/
    static void test3() throws Exception {
        SimpleMatrix A = new SimpleMatrix(a);
        SimpleMatrix B = new SimpleMatrix(b);
        SimpleMatrix x = DGELSX(A,B);
        x.print(8,3);
    }

    static void test4() throws Exception {
        logger.info("Test 4");
        double[][] a = {
                {1,0,0,0,0,0},
                {0,1,0,0,0,0},
                {0,0,0,0,1,0},
                {0,0,0,0,0,1},
                {1,0,1,1,1,0},
                {0,1,1,1,1,0},
                {1,0,1,1,0,1},
                {0,1,1,1,0,1},
            };
        double[][] b = {
                {10.001},
                {15.002},
                {30.001},
                {35.002},
                {85.001},
                {90.002},
                {90.001},
                {95.002},
            };

        double[][] a2 = {
                {1,0,0,0,0,0},
                {0,0,0,0,1,0},
                {0,0,1,0,0,0},
                {0,0,0,1,0,0},
                {1,1,0,1,0,1},
                {0,1,1,0,1,1},
                {1,1,0,0,1,1},
                {0,1,1,1,0,1},
            };
        double[][] b2 = {
                {10.001},
                {35.002},
                {15.002},
                {30.001},
                {85.001},
                {95.002},
                {90.001},
                {90.002},
            };
        SimpleMatrix A = new SimpleMatrix(a);
        SimpleMatrix B = new SimpleMatrix(b);
//        Matrix x = NNLS.NNLS(A,B);
//        x.print(8,3);
        testMatrix(A,B);

        A = new SimpleMatrix(a2);
        B = new SimpleMatrix(b2);
        testMatrix(A,B);
//        x = NNLS.NNLS(A,B);
//        x.print(8,3);


    }


	public static void test5() throws Exception {
		double[][] a5 = {
				{1,1,0,0},
				{1,0,1,0},
				{1,0,0,1},
				{1,1,0,0},
				{0,1,1,0},
				{0,1,0,1},
				{1,0,1,0},
				{0,1,1,0},
				{0,0,1,1},
				{1,0,0,1},
				{0,1,0,1},
				{0,0,1,1}
		};
		double[][] b5 = {{24.3},{28.1},{43.2},{27.3},{33.8},{43.8},{31.7},{37.6},{48.4},{38.9},{47.4},{51.9}};

		int m = 12;
		int n = 4;

		SimpleMatrix A = new SimpleMatrix(a5);
		SimpleMatrix B = new SimpleMatrix(b5);

//                Jama.QRDecomposition qr = new Jama.QRDecomposition(simpleToJama(Ap));
//                SimpleMatrix z = jamaToSimple(qr.solve(simpleToJama(B)));
                // This overwrites Ap
        SimpleMatrix z = new SimpleMatrix(m,1);
		for(int j=0; j<m; j++) {
			z.a[j][0] = B.a[j][0];
		}
        int[] JPVT = new int[n];
//                for(int j=0; j<n; j++) {
//                    JPVT[j] = Z[j]?0:1;
//                }
		org.netlib.util.intW RANK = new org.netlib.util.intW(0);
		double[] work = new double[(Math.max( Math.min(m,n)+3*n, 2*Math.min(m,n)+1 ))];
		org.netlib.util.intW INFO = new org.netlib.util.intW(0);
		org.netlib.lapack.DGELSX.DGELSX(m, n, 1, A.a, z.a, JPVT, 1E-15, RANK, work, INFO);

		logger.info("RANK: "+RANK.val);
		if(INFO.val<0) logger.severe(INFO.val+"th argument had an illegal value");

		logger.info(z.toString(8,2));
		
		Matrix x = NNLS.NNLS(A,B);
		logger.info(x.toString(8,2));
	}

    public static void initLogging() {
        Logger root = Logger.getLogger("");
        Handler[] old = root.getHandlers();
        for(int i=0; i<old.length; i++) {
            root.removeHandler(old[i]);
        }
        Handler handler = new ConsoleHandler();
        handler.setFormatter(new Formatter() {
            public String format(LogRecord record) {
                return record.getMessage()+"\n";
            }
        });
        root.addHandler(handler);
        root.setLevel(Level.ALL);
    }

    public void run(String[] args) {
        initLogging();
        String arg1 = "";

        try {
            if (args.length >= 2) arg1 = args[1];
            String test = args[0];
            int num;

            try {
                num = Integer.parseInt(test);
            } catch (NumberFormatException e) {
                num = 1;
            }
            logger.info("Doing test #" + num);
            Method method = this.getClass().getDeclaredMethod("test"+num, null);
            method.invoke(this, null);
        } catch (Throwable t) {
            t.printStackTrace();
        }
        System.exit(0);
    }

    public static void main(String[] args) {
        Test test = new Test();
        test.run(args);
    }
}

